@@ -28,6 +28,9 @@ class Database:
             self.execute("create database " + self.database)
             self.conn = mysql.connector.connect(**self.kwargs)
 
+        self.execute('SET NAMES utf8mb4')
+        self.execute('SET CHARACTER SET utf8mb4')
+
     @property
     def kwargs(self):
         return dict(user=self.user, password=self.password, host=self.host, database=self.database, charset=self.charset)
@@ -131,7 +134,162 @@ class MySQLConnector(Database):
         Database.__init__(self)
         
     def load_data_from_list(self, table, array, step=10000, replace=False, ignore=True, truncate=False):
-        desc = self.desc_table(table)
+        try:
+            desc = self.desc_table(table)
+        except mysql.connector.errors.ProgrammingError as err:
+            print(err)
+            m = re.search("1146 \(42S02\): Table '(\w+).(\w+)' doesn't exist", str(err))
+            database, table = m.groups()
+            print('database =', database)
+            print('table =', table)
+            
+            keys = set()
+            for obj in array:
+                keys |= obj.keys()
+            
+            keys = [*keys]
+            dtype = std.Object()
+            for obj in array:
+                for key in keys:
+                    match obj.get(key):
+                        case None:
+                            ...
+                    
+                        case bool():
+                            if dtype[key] is None:
+                                dtype[key] = bool
+                        
+                        case int():
+                            if dtype[key] is None or dtype[key] is bool:
+                                dtype[key] = int
+
+                        case float():
+                            if dtype[key] is None or dtype[key] is bool or dtype[key] is int:
+                                dtype[key] = float
+
+                        case str():
+                            if dtype[key] is None or dtype[key] is bool or dtype[key] is int or dtype[key] is float:
+                                dtype[key] = str
+            
+                        case list() | tuple() | set() | dict():
+                            if dtype[key] is None or dtype[key] is bool or dtype[key] is int or dtype[key] is float or dtype[key] is str:
+                                dtype[key] = object
+                                
+                        case _:
+                            raise _
+            
+            PRI = None
+
+            def detect_primary_key(array, key):
+                primary_keys = set()
+                for obj in array:
+                    if pk := obj.get(key):
+                        if pk in primary_keys:
+                            break
+                        else:
+                            if isinstance(pk, str):
+                                try:
+                                    obj = json.loads(pk)
+                                    return
+                                except:
+                                    ...
+                            primary_keys.add(pk)
+                    else:
+                        break
+                else:
+                    return key
+                
+            desc = {}
+            for key in keys:
+                match dtype[key].__name__:
+                    case 'bool':
+                        desc[key] = 'tinyint'
+
+                    case 'int':
+                        intvals = []
+                        for obj in array:
+                            if s := obj.get(key):
+                                intvals.append(s)
+                            else:
+                                intvals.append(0)
+
+                        if min(intvals) >= 0 and max(intvals) < 256:
+                            desc[key] = 'tinyint'
+                        else:
+                            desc[key] = 'int'
+
+                        if PRI is not None:
+                            continue
+
+                        PRI = detect_primary_key(array, key)
+                        
+                    case 'float':
+                        desc[key] = 'float'
+                        
+                    case 'str':
+                        length = 0
+                        for obj in array:
+                            if s := obj.get(key):
+                                length = max(len(s), length)
+
+                        if length >= 768:
+                            desc[key] = 'text'
+                        else:
+                            desc[key] = f'varchar({length})'
+                            if PRI is not None:
+                                continue
+    
+                            PRI = detect_primary_key(array, key)
+                    case 'object':
+                        desc[key] = 'json'
+                        
+                    case _:
+                        raise _
+
+            if PRI is None:
+                if 'id' not in keys:
+                    PRI = 'id'
+                elif 'uuid' not in keys:
+                    PRI = 'uuid'
+                elif '_id' not in keys:
+                    PRI = '_id'
+
+                for obj in array:
+                    obj[PRI] = 0
+                    
+                keys.insert(0, PRI)
+                desc[PRI] = 'int not null auto_increment'
+            else:
+                desc[PRI] += ' not null' 
+
+            def sort_key(key):
+                if key == PRI:
+                    return 0, len(key), key
+
+                if key == 'training':
+                    return 3, len(key), key
+                
+                if dtype[key] is str:
+                    if desc[key] == 'text':
+                        return 1, 65536, key
+
+                    m = re.search('\d+', desc[key])
+                    return 1, int(m[0]), key
+                         
+                return 2, len(key), key
+
+            keys.sort(key=sort_key)
+            print(keys)
+
+            desc = ',\n'.join(map(lambda key : f"`{key}` {desc[key]}", keys)) 
+            sql_create_table = f"CREATE TABLE `{database}`.`{table}` ({desc}, PRIMARY KEY (`{PRI}`)) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci PARTITION BY KEY() PARTITIONS 8"
+            print(sql_create_table)
+            try:
+                self.execute(sql_create_table)
+            except Exception as e:
+                print(e)
+                raise e
+            desc = self.desc_table(table)
         
         has_training_field = False
         
@@ -167,39 +325,51 @@ class MySQLConnector(Database):
                         if isinstance(args, tuple):
                             args = [*args]
                         elif isinstance(args, dict):
-                            args = [args.get(Field, '') for Field, *_ in desc]
+                            args = [args.get(Field) for Field, *_ in desc]
                             
                         for i, arg in enumerate(args):
                             if isinstance(arg, set):
                                 arg = [*arg]
-                                                            
-                            if isinstance(arg, (list, tuple)):
-                                arg = std.json_encode(arg)
-                                arg = std.json_encode(arg)[1:-1]
+
+                            if isinstance(arg, (list, tuple, dict)):
+                                cbytes = std.json_encode(std.json_encode(arg))[1:-1]
                             elif isinstance(arg, str): 
-                                arg = std.json_encode(arg)[1:-1]
+                                cbytes = std.json_encode(arg)[1:-1]
                             elif isinstance(arg, bytes):
                                 if len(arg) > char_length[i]:
                                     print(args)
                                     print('args[%d] exceeds the allowable length %d' % (i, char_length[i]))
                                     arg = arg[:char_length[i]]
                                     
-                                arg = arg.decode()
-                            elif isinstance(arg, dict):
-                                arg = std.json_encode(std.json_encode(arg))[1:-1]
+                                cbytes = arg.decode()
+                            elif arg is None:
+                                cbytes = 'null'
                             else:
-                                arg = str(arg)
-                            
-                            if not ignore and (len(arg.encode(encoding='utf8')) if dtype[i] == 'text' else len(arg)) > char_length[i]:
-                                if truncate:
-                                    print('truncating the data to maximum length:', char_length[i], ", since its length is", len(arg.encode(encoding='utf8')))
-                                    arg = arg[:char_length[i]]
+                                cbytes = str(arg)
+
+                            if not ignore:
+                                if is_varchar(dtype[i]):
+                                    if len(arg) > char_length[i]:
+                                        if truncate:
+                                            print('truncating the data to maximum length:', char_length[i], ", since its length is", len(arg))
+                                            arg = arg[:char_length[i]]
+                                            cbytes = std.json_encode(arg)[1:-1]
+                                        else:
+                                            print(args)
+                                            print('args[%d] exceeds the allowable length %d, since its length is %d' % (i, char_length[i], len(arg)))
+                                            args = None
+                                            break
                                 else:
-                                    print(args)
-                                    print('args[%d] exceeds the allowable length %d' % (i, char_length[i]))
-                                    args = None
-                                    break
-                            args[i] = arg
+                                    if len(cbytes) > char_length[i]:
+                                        if truncate:
+                                            print('truncating the data to maximum length:', char_length[i], ", since its length is", len(cbytes))
+                                            cbytes = cbytes[:char_length[i]]
+                                        else:
+                                            print(args)
+                                            print('args[%d] exceeds the allowable length %d' % (i, char_length[i]))
+                                            args = None
+                                            break
+                            args[i] = cbytes
                         
                         if args:
                             if has_training_field and len(args) < len(desc):
@@ -214,20 +384,59 @@ class MySQLConnector(Database):
             rowcount += self.load_data_from_csv(table, csv, delete=True, replace=replace, ignore=ignore)
         return rowcount
 
-    def load_data(self, table, data, **kwargs):
-        if isinstance(data, str):
-            return self.load_data_from_csv(table, data, **kwargs)
-        return self.load_data_from_list(table, data, **kwargs)
+    def load_data(self, table, *args, **kwargs):
+        if args:
+            arg, = args
+            if isinstance(arg, str):
+                return self.load_data_from_csv(table, arg, **kwargs)
+            return self.load_data_from_list(table, arg, **kwargs)
+        else:
+            table_name = re.sub('\W', '_', table.lower())
+            [[count]] = self.query(f"select count(*) from information_schema.tables WHERE table_name = '{table_name}'")
+            if count:
+                print(table, 'already exists, skipping')
+                return
             
+            from datasets import load_dataset
+            try:
+                data = load_dataset(table)
+            except ValueError as e:
+                err = str(e)
+                print(err)
+                if m:= re.search("`load_dataset\('(\S+)', '(\S+)'\)`", err):
+                    assert m[1] == table
+                    config = m[2]
+                    data = load_dataset(table, config)
+                else:
+                    raise e
+
+            array = []
+            for key in data:
+                key = key.lower()
+                if key.startswith('train'):
+                    training = 1
+                elif key.startswith('test'):
+                    training = 0
+                elif key.startswith('dev') or key.startswith('val'):
+                    training = 2
+                else:
+                    print('unrecognized key', key, 'from', data.keys())
+                    raise
+                
+                for obj in data[key]:
+                    obj['training'] = training
+                    array.append(obj)
+            return self.load_data(table_name, array)
+
     def load_data_from_csv(self, table, csv, delete=False, replace=False, ignore=False):
         start = time.time()
         csv = csv.replace('\\', '/')
         if replace:
-            sql = 'load data local infile "%s" replace into table %s character set utf8' % (csv, table)
+            sql = 'load data local infile "%s" replace into table %s character set utf8mb4' % (csv, table)
         elif ignore:
-            sql = 'load data local infile "%s" ignore into table %s character set utf8' % (csv, table)
+            sql = 'load data local infile "%s" ignore into table %s character set utf8mb4' % (csv, table)
         else:
-            sql = 'load data local infile "%s" into table %s character set utf8' % (csv, table)
+            sql = 'load data local infile "%s" into table %s character set utf8mb4' % (csv, table)
         print('executing: ', sql)
         
         local_infile = True
@@ -393,7 +602,7 @@ def is_float(Type):
     return re.match('double', Type)
 
 
-def is_string(Type):
+def is_varchar(Type):
     return re.match('varchar\(\d+\)', Type)
 
 
