@@ -834,6 +834,74 @@ def array_split(self, pivot):
             latter.append(arg)
     return former, latter
 
+def parse_args(argv, args, kwargs):
+    # print('in parse_args')
+    # print('argv =', argv)
+    # print('args =', args)
+    # print('kwargs =', kwargs)
+    
+    for i, arg in enumerate(argv):
+        if arg.startswith('--'):
+            args.extend(argv[:i]) 
+            return parse_kwargs(argv[i:], args, kwargs)
+    else:
+        args.extend(argv)
+
+def parse_kwargs(argv, args, kwargs):
+    # print('in parse_kwargs')
+    # print('argv =', argv)
+    # print('args =', args)
+    # print('kwargs =', kwargs)
+    def get_val(value):
+        try:
+            return eval(value)
+        except:
+            return value
+        
+    name = None
+    for i, arg in enumerate(argv):
+        if arg.startswith('--'):
+            if name is not None:
+                if kwargs[name] is None:
+                    kwargs[name] = True
+
+            name = arg[2:]
+            if '=' in name:
+                index = name.index('=')
+                name, arg = name[:index], name[index + 1:]
+                kwargs[name] = get_val(arg)
+                name = None
+        elif name is None:
+            return parse_args(argv[i:], args, kwargs)
+
+        elif name in kwargs:
+            if not isinstance(kwargs[name], list):
+                kwargs[name] = [kwargs[name]]
+            kwargs[name].append(get_val(arg))
+        else:
+            kwargs[name] = get_val(arg)
+
+    if name is not None:
+        if kwargs[name] is None:
+            kwargs[name] = True
+
+    return kwargs
+    
+def argparse():
+    argv = sys.argv[1:]
+    print('argv =', argv)
+
+    args = []
+    kwargs = Object()
+    parse_args(argv, args, kwargs)
+    return args, kwargs
+
+def kill():
+    pid = os.getpid()
+    if is_Linux():
+        os.system(f"kill -9 {pid}")
+    else:
+        os.system(f"taskkill /F /pid {pid}")
 
 if __name__ == '__main__':
     arr = [*range(10)]
