@@ -14,7 +14,7 @@ ctypes2numpy = {"?": bool,
           "H": np.ushort,
           "i": np.int32,
           "I": np.uint32,
-          "l": np.long,
+          "l": np.int64,
           "q": np.longlong,
           "Q": np.ulonglong,
           }
